-- Chaos Frame Modifier
-- Author: ChaosInc
-- (c) 2009-2011 Joshua Mrozek , All rights reserved.

-- BLIZZARD UI LIST
local BlizzUI = {
	"Achievement",
	"Archaeology",
	"Arena",
	"Auction",
	"BattlefieldMinimap",
	"Calendar",
	"GuildBank",
	--"GuildControl",
	"GuildUI",
	"ItemSocketing",
	"Reforging",
	"TalentUI",
	"TimeManager",
	"Vehicle",
}

-- Load Blizzard UIs so they finally stop causing me headaches
for i,k in ipairs(BlizzUI) do
	LoadAddOn("Blizzard_".. BlizzUI[i].. "UI")
end

AchievementAlertFrame_GetAlertFrame()  -- load achievement alert frames

-- DEFAULT FRAMES
local defaults = {
	"PlayerFrame",
	"TargetFrame",
	"Minimap",
	"MinimapCluster",
	"ChatFrame1",
	"MainMenuBar",
	"TimeManagerClockButton",
	"FriendsMicroButton",
	"WorldFrame"
	}
	
selName = ""

--[[ SLASH COMMANDS ]]
SLASH_CFM1 = "/cfm";
SlashCmdList["CFM"] = function() CFM_GUI() end

--[[ VIRTUAL FRAME OBJECT]]
CFM = CFM or {}
local CFM = CreateFrame("Frame")
CFM:SetScript("OnEvent", function(self, event, ...) if CFM[event] then return CFM[event](self, event, ...) end end)
CFM:RegisterEvent("ADDON_LOADED")
CFM:RegisterEvent("PLAYER_ENTERING_WORLD")
CFM:RegisterEvent("UNIT_EXITED_VEHICLE")
CFM:RegisterEvent("UNIT_ENTERED_VEHICLE")
CFM:RegisterEvent("PLAYER_TALENT_CHANGED")
CFM:Show()

--[[ ADDON_LOADED ]]
function CFM:ADDON_LOADED(_,addon)
	if addon ~= "CFM" then return end

	-- pull info for profile settings
	local realm, toon
	toon = UnitName("player")
	realm = GetRealmName()

	-- check for saved vars or create new
	-- global profiles
	CFM_Profiles = CFM_Profiles or {}
	CFM_Profiles[realm] = CFM_Profiles[realm] or {}
	CFM_Profiles[realm][toon] = CFM_Profiles[realm][toon] or {}
	
	-- per character
	CFM_Settings = CFM_Settings or {}
	CFM_Settings.useProfile = CFM_Settings.useProfile or {realm, toon}
	CFM_Settings.border = CFM_Settings.border or "Interface\\ACHIEVEMENTFRAME\\UI-Achievement-WoodBorder"
	CFM_Settings.bg = CFM_Settings.bg or "Interface\\ACHIEVEMENTFRAME\\UI-Achievement-StatsBackground"
	
	-- shorthand that long ass reference
	activeProfile = CFM_Profiles[CFM_Settings.useProfile[1]][CFM_Settings.useProfile[2]] or CFM_Profiles[realm][toon]
	
	-- prevent closing of interface frame from readjusting frames
	InterfaceOptionsFrame:HookScript("OnHide", function() for k,_ in pairs(activeProfile) do CFM_ApplySettings(k) end end)
	
	-- change old restore data from settings to frame profile
	if CFM_Settings.restore then
		for k,_ in pairs(CFM_Settings.restore) do
			activeProfile[k].restore = {}
			activeProfile[k].restore[k] = {}
			activeProfile[k].restore[k] = CopyTable(CFM_Settings.restore[k])
		end
		CFM_Settings.restore = nil
	end
	
	-- add ".frame" to convert variable format to 'frame-keys' to 'name-keys' from older versions
	for k,_ in pairs(activeProfile) do
		if (not activeProfile[k].frame) then
			activeProfile[k.."Temp"] = CopyTable(activeProfile[k])
			activeProfile[k.."Temp"].frame = k
			-- convert restore data
			activeProfile[name].restore[k.."Temp"] = CopyTable(activeProfile[name].restore[k])
			-- nuke the old stuff
			activeProfile[name].restore[k] = nil
			activeProfile[k] = nil
		end

	end	
end

--[[ PLAYER_ENTERING_WORLD ]]
function CFM:PLAYER_ENTERING_WORLD()
	-- ALL ADDONS LOADED, NOW HOOK REGISTERED FRAMES "OnShow" SCRIPTS
	for k,_ in pairs(activeProfile) do
		local frame = activeProfile[k].frame
		_G[frame]:HookScript("OnShow", function() CFM_ApplySettings(k) end)
	end
	-- wait before applying position changes just to make sure
	self.delay = 2
	self:SetScript("OnUpdate", function(self, elapsed) CFM["OnUpdate"](self, elapsed) end)
end

--[[ UNIT_ENTERED_VEHICLE ]]  -- fix for animation screwing up positioning of vehicle frame
function CFM:UNIT_ENTERED_VEHICLE()
	self.delay = .5
	self:SetScript("OnUpdate", function(self, elapsed) CFM["OnUpdate"](self,elapsed) end)
end

--[[ UNIT_EXITED_VEHICLE ]]
function CFM:UNIT_EXITED_VEHICLE(_, unit)
	if unit == "player" then
		for k,_ in pairs(activeProfile) do
			CFM_ApplySettings(k)
		end
	end
end

--[[ PLAYER_TALENT_CHANGED ]]
function CFM:PLAYER_TALENT_CHANGED()
	for k,_ in pairs(activeProfile) do
		CFM_ApplySettings(k)
	end
end

--[[ PLAYER_REGEN_ENABLED ]]
function CFM:PLAYER_REGEN_ENABLED()
	for k,_ in pairs(activeProfile) do
		CFM_ApplySettings(k)
	end	
	-- print("|c00FF00FFCFM:|r Combat ended.  Settings now applied.")
	self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	CFM.waiting = false
end
	
--[[ ONUPDATE ]]
function CFM:OnUpdate(elapsed)
	if self.delay ~= nil then
		self.delay = self.delay - elapsed
		if self.delay < 0 then
			self.delay = nil
			for k,_ in pairs(activeProfile) do
				if _G[activeProfile[k].frame] == nil then
					-- do nothing
				else
					CFM_ApplySettings(k)
				end
			end
			self:SetScript("OnUpdate", nil)
		end
	end
end

StaticPopupDialogs["CFM_PROMPTREMOVE"] = {
  text = "Remove %s from the database?",
  button1 = "Yes",
  button2 = "No",
  OnAccept = function()
      CFM_RemoveFrame(remFrame)
	  remFrame = nil
  end,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1
};

StaticPopupDialogs["CFM_PROMPTRELOAD"] = {
  text = "Reload UI? (may not be necessary)",
  button1 = "Yes",
  button2 = "No",
  OnAccept = function()
	ReloadUI()
  end,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1
}

StaticPopupDialogs["CFM_PROMPTDEFAULTS"] = {
  text = "Load Default Frames?",
  button1 = "Yes",
  button2 = "No",
  OnAccept = function()
	for _,frame in ipairs(defaults) do
		-- make the name the same as the frame name with spaces
		local name = gsub(frame, "([a-z]+)([A-Z][a-z]+)", "%1 %2")
		CFM_AddFrame(name, frame)
	end
  end,
  timeout = 0,
  whileDead = 1,
  hideOnEscape = 1
}

--[[ ADD NEW FRAME ]]
function CFM_AddFrame(name, frame, userparent)
	if frame == "" then
		return
	end
	-- check if frame is already registered or non-existant
	for _,v in pairs(activeProfile) do
		if (v == frame) then
			print("|c00FF00FFCFM:|r ".. frame.. " is already in the database.")
			return
		elseif _G[frame] == nil then
			print("|c00FF00FFCFM:|r Frame ".. frame.. " doesn't exist.  Please check your spelling.")
			return
		end
	end
	
	if name == "" then name = frame end -- default to physical name if user doesn't provide one
	
	-- begin saving the frame and it's properties under it's name
	activeProfile[name] = {}
	activeProfile[name].frame = frame
	if userparent == nil then
		userparent = "UIParent"
	end
	activeProfile[name].parent = userparent
	CFM_SaveFrame(name, frame)
	
	-- save restore settings
	activeProfile[name].restore = activeProfile[name].restore or {}
	activeProfile[name].restore = CopyTable(activeProfile[name])


	activeProfile[name].forceHide = false
	activeProfile[name].sameXY = false
	activeProfile[name].lock = true
	_G[frame]:HookScript("OnShow", function() CFM_ApplySettings(name) end)
	
	print(frame.. " added.")
	CFM_ScrollUpdate()
end

--[[ RESTORE SETTINGS ]]
function CFM_RestoreFrame(name)
	-- message for a reload in case restore data doesn't exist
	if activeProfile[name].restore == nil then
		print("Restore data for ".. name.. " is either missing or corrupt.  Skipping this step.")
		return
	end
	
	-- temporarily redirect
	local tempRestore = CopyTable(activeProfile[name].restore)
	local tempProfile = CopyTable(activeProfile[name])
	activeProfile[name] = CopyTable(activeProfile[name].restore)
	CFM_ApplySettings(name)
		
	-- restore directory and restore lost restore data
	activeProfile = CFM_Profiles[CFM_Settings.useProfile[1]][CFM_Settings.useProfile[2]]
	activeProfile[name] = CopyTable(tempProfile)
	activeProfile[name].restore = CopyTable(tempRestore)

end

--[[ REMOVE FRAME ]]
function CFM_RemoveFrame(name)
	local frame = activeProfile[name].frame

	CFM_RestoreFrame(name)
	
	for i,k in ipairs(activeProfile) do
		if k == name then
			tremove(activeProfile, i)
		end
	end
	print(frame.. " has been removed.  Previous settings restored.")
	activeProfile[name] = nil
	-- clear leftover data from prop frame
	CFM_XBox:SetText("")
	CFM_YBox:SetText("")
	CFM_WidthBox:SetText("")
	CFM_HeightBox:SetText("")
	CFM_LevelBox:SetText("")
	CFM_ScaleBox:SetText("")
	CFM_ChangeParentBox:SetText("")
	CFM_AlphaBox:SetText("")
	CFM_ScrollUpdate()
end

--[[ SAVE FRAME ]]
function CFM_SaveFrame(name, frame)
	frame = _G[frame] -- typing _G[] over and over is annoying
	if (not frame) then return end  -- do nothing if the frame isn't present

	-- pull the frame's current info and save it to our profile
	local point, parent, relativePoint, offsetX, offsetY = frame:GetPoint()
	if (parent == nil) then
		activeProfile[name].parent = "UIParent"  -- some people are really bad at setting parent frames
	else
		activeProfile[name].parent = parent:GetName()
	end
	
	activeProfile[name].point = point
	activeProfile[name].relativePoint = relativePoint
	activeProfile[name].offsetX = offsetX
	activeProfile[name].offsetY = offsetY
	activeProfile[name].width = frame:GetWidth()
	activeProfile[name].height = frame:GetHeight()
	activeProfile[name].scale = frame:GetScale()
	activeProfile[name].level = frame:GetFrameLevel()
	activeProfile[name].alpha = frame:GetAlpha()
	activeProfile[name].clamp = frame:IsClampedToScreen()
	activeProfile[name].mouseEnabled = frame:IsMouseEnabled()
	if frame:GetFrameStrata() == "UNKNOWN" then
		activeProfile[name].strata = "Background"  -- workaround for WorldFrame
	else
		activeProfile[name].strata = frame:GetFrameStrata()
	end
end

--[[ APPLY FRAME SETTINGS ]]
function CFM_ApplySettings(name)
	local frame = activeProfile[name].frame
	if activeProfile[name] == nil or _G[frame] == nil then print(frame.. " not found.") return end
	-- apply frame's settings
	if not UnitAffectingCombat("player") then
		_G[frame]:SetMovable(true)
		_G[frame]:EnableMouse(activeProfile[name].mouseEnabled)
		_G[frame]:ClearAllPoints()
		_G[frame]:SetPoint(activeProfile[name].point, activeProfile[name].parent, activeProfile[name].relativePoint, format("%d", activeProfile[name].offsetX), format("%d", activeProfile[name].offsetY))	
		_G[frame]:SetClampedToScreen(false) -- temp chat box fix
		if frame == "WorldFrame" and (activeProfile[name].strata ~= "BACKGROUND") then
			activeProfile[name].strata = "BACKGROUND"
			print("|c00FF00FFCFM:|r WorldFrame strata restored to 'Background'.  Strata changes to this frame are now prevented for user safety.")
		end
		_G[frame]:SetFrameStrata(activeProfile[name].strata)
		_G[frame]:SetFrameLevel(activeProfile[name].level)
		_G[frame]:SetScale(activeProfile[name].scale)
		_G[frame]:SetHeight(format("%d", activeProfile[name].height))
		_G[frame]:SetWidth(format("%d", activeProfile[name].width))
		
		-- make sure alpha doesn't go below 0 or above 1
		if activeProfile[name].alpha < 0 then activeProfile[name].alpha = 0 end
		if activeProfile[name].alpha > 1 then activeProfile[name].alpha = 1 end
		
		_G[frame]:SetAlpha(activeProfile[name].alpha)
		if _G[activeProfile[name].parent] == nil then
			activeProfile[name].parent = "UIParent"
		end
		if activeProfile[name].forceHide then
			_G[frame]:ClearAllPoints()
			_G[frame]:SetPoint("BOTTOMLEFT", UIParent, "TOPLEFT", 0, 0)
			_G[frame]:Hide()
		end
	else
		if not CFM.waiting then
			-- print("|c00FF00FFCFM:|r Settings will be applied once you leave combat.")
			CFM:RegisterEvent("PLAYER_REGEN_ENABLED")
			CFM.waiting = true  -- spam prevention
		end
	end
end	

--[[ MOVER FUNCTION ]]
function CFM_MoveFrame(name, dir)
	if dir == "left" then
		activeProfile[name].offsetX = activeProfile[name].offsetX -1
	elseif dir == "right" then
		activeProfile[name].offsetX = activeProfile[name].offsetX +1
	elseif dir == "up" then
		activeProfile[name].offsetY = activeProfile[name].offsetY +1
	elseif dir == "down" then
		activeProfile[name].offsetY = activeProfile[name].offsetY -1
	end
	CFM_XBox:SetText(activeProfile[name].offsetX)
	CFM_YBox:SetText(activeProfile[name].offsetY)	
	CFM_ApplySettings(name)
end

function CFM_StopMoving(name, frame)
	if frame.isMoving then
		frame:StopMovingOrSizing()
		local a,_,b,postX, postY = frame:GetPoint()
		activeProfile[name].point = a
		activeProfile[name].relativePoint = b
		activeProfile[name].offsetX = postX
		activeProfile[name].offsetY = postY
		CFM_ApplySettings(name)
		CFM_XBox:SetText(activeProfile[name].offsetX)
		CFM_YBox:SetText(activeProfile[name].offsetY)
		frame.isMoving = false
		if  a == "CENTER" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 0)
		elseif a == "TOP" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 1)
		elseif a == "TOPRIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 2)
		elseif a == "RIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 3)
		elseif a == "BOTTOMRIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 4)
		elseif a == "BOTTOM" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 5)
		elseif a == "BOTTOMLEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 6)
		elseif a == "LEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 7)
		elseif a == "TOPLEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_FromBox, 8)
		end
		UIDropDownMenu_SetText(CFM_FromBox, a)
		if b == "CENTER" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 0)
		elseif b == "TOP" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 1)
		elseif b == "TOPRIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 2)
		elseif b == "RIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 3)
		elseif b == "BOTTOMRIGHT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 4)
		elseif b == "BOTTOM" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 5)
		elseif b == "BOTTOMLEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 6)
		elseif b == "LEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 7)
		elseif b == "TOPLEFT" then
			UIDropDownMenu_SetSelectedValue(CFM_ToBox, 8)
		end
		UIDropDownMenu_SetText(CFM_ToBox, b)
	else
		return
	end 
end

function CFM_GetPoints(name)
	local point, parent, relativePoint, offsetX, offsetY
	point = activeProfile[name].point
	parent = activeProfile[name].parent
	relativePoint = activeProfile[name].relativePoint
	offsetX = activeProfile[name].offsetX
	offsetY = activeProfile[name].offsetY
	return name, point, parent, relativePoint, offsetX, offsetY
end

